@@ -109,6 +109,11 @@ typedef struct {
     ESYS_TR flushHandle;
 } FlushContext_IN;
 
+typedef struct {
+    ESYS_TR pcrHandle;
+    TPM2B_AUTH authData;
+} PCR_IN;
+
 /** Union for input parameters.
  *
  * The input parameters of a command need to be stored if they are needed
@@ -130,6 +135,7 @@ typedef union {
     Policy_IN Policy;
     NV_IN NV;
     FlushContext_IN FlushContext;
+    PCR_IN PCR;
 } IESYS_CMD_IN_PARAM;
 
 /** The states for the ESAPI's internal state machine */
@@ -182,6 +188,11 @@ struct ESYS_CONTEXT {
                                       automatically loaded. */
     IESYS_SESSION *enc_session;  /**< Ptr to the enc param session.
                                       Used to restore session attributes */
+    ESYS_TR sav_session1;        /**< Used to store session for cases where call
+                                      with ESYS_TR_NONE is needed to determine object
+                                      name */
+    ESYS_TR sav_session2;
+    ESYS_TR sav_session3;
 };
 
 /** The number of authomatic resubmissions.
