@@ -210,7 +210,11 @@ cleanup:
 
 /** Get public data of a NV object from keystore.
  *
- * @param[in] path The relative path of the NV object.
+ * @param[in] path The relative path of the NV object. The path will
+              be used to get the public from the keystore if no nv_index is
+              passed.
+ * @param[in] nv_index The index of the NV object. The index will be used to
+ *            get the public data if nv_index > 0.
  * @param[out] nv_public The caller allocated public structure.
  * @param[in,out] ctx The context to access io and keystore module and to store
  *                the io state.
@@ -238,42 +242,79 @@ cleanup:
 TSS2_RC
 ifapi_get_nv_public(
     const char *path,
+    TPMI_RH_NV_INDEX nv_index,
     TPM2B_NV_PUBLIC *nv_public,
     void *ctx)
 {
     TSS2_RC r = TSS2_RC_SUCCESS;
     IFAPI_OBJECT object;
     FAPI_CONTEXT *context = ctx;
+    TPM2B_NV_PUBLIC *nv_public_esys;
+    ESYS_TR esys_tr;
 
-    switch (context->io_state) {
-    statecase(context->io_state, IO_INIT)
-        /* Prepare the loading of the NV object. */
-        r = ifapi_keystore_load_async(&context->keystore, &context->io, path);
-        return_if_error2(r, "Could not open: %s", path);
-        fallthrough;
+    if (nv_index) {
+        switch (context->read_nv_public_state) {
+            statecase(context->read_nv_public_state, READ_NV_PUBLIC_INIT)
+                r = Esys_TR_FromTPMPublic_Async(context->esys, nv_index, ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE);
+                goto_if_error(r, "Error: tr from public", cleanup);
+                fallthrough;
 
-    statecase(context->io_state, IO_ACTIVE)
-        /* Finalize or retry the reading and check the object type */
-        r = ifapi_keystore_load_finish(&context->keystore, &context->io,
-                                       &object);
-        return_try_again(r);
-        return_if_error(r, "read_finish failed");
+            statecase(context->read_nv_public_state, READ_NV_PUBLIC_GET_ESYS_TR)
+                r = Esys_TR_FromTPMPublic_Finish(context->esys, &esys_tr);
+                try_again_or_error_goto(r, "Error: tr from public finish", cleanup);
 
-        if (object.objectType != IFAPI_NV_OBJ) {
-            goto_error(r, TSS2_FAPI_RC_BAD_VALUE, "Object %s is not a key.",
-                       cleanup, path);
+                r = Esys_NV_ReadPublic_Async(context->esys, esys_tr,
+                                             ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE);
+                goto_if_error(r, "Error: nv read public async", cleanup);
+                fallthrough;
+
+            statecase(context->read_nv_public_state, READ_NV_PUBLIC_GET_PUBLIC)
+                r = Esys_NV_ReadPublic_Finish(context->esys,
+                                              &nv_public_esys,
+                                              NULL);
+                try_again_or_error_goto(r, "Error: nv read public finish", cleanup);
+
+                *nv_public = *nv_public_esys;
+                SAFE_FREE(nv_public_esys);
+                context->io_state = IO_INIT;
+                break;
+
+            statecasedefault(context->state);
         }
 
-        *nv_public = object.misc.nv.public;
-        context->io_state = IO_INIT;
-        break;
+    } else {
+        switch (context->io_state) {
+            statecase(context->io_state, IO_INIT)
+                /* Prepare the loading of the NV object. */
+                r = ifapi_keystore_load_async(&context->keystore, &context->io, path);
+                return_if_error2(r, "Could not open: %s", path);
+                fallthrough;
 
-    statecasedefault(context->state);
+            statecase(context->io_state, IO_ACTIVE)
+                /* Finalize or retry the reading and check the object type */
+                r = ifapi_keystore_load_finish(&context->keystore, &context->io,
+                                               &object);
+                return_try_again(r);
+                return_if_error(r, "read_finish failed");
+
+                if (object.objectType != IFAPI_NV_OBJ) {
+                    goto_error(r, TSS2_FAPI_RC_BAD_VALUE, "Object %s is not a key.",
+                               cleanup, path);
+                }
+
+                *nv_public = object.misc.nv.public;
+                context->io_state = IO_INIT;
+                break;
+
+            statecasedefault(context->state);
+        }
     }
 
 cleanup:
     context->io_state = IO_INIT;
-    ifapi_cleanup_ifapi_object(&object);
+    if (!nv_index) {
+        ifapi_cleanup_ifapi_object(&object);
+    }
     return r;
 }
 
@@ -451,7 +492,6 @@ ifapi_policyeval_cbauth(
     FAPI_CONTEXT *fapi_ctx = userdata;
     IFAPI_POLICY_EXEC_CTX *current_policy;
     IFAPI_POLICY_EXEC_CB_CTX *cb_ctx;
-    bool next_case;
 
     return_if_null(fapi_ctx, "Bad user data.", TSS2_FAPI_RC_BAD_REFERENCE);
     return_if_null(fapi_ctx->policy.policyutil_stack, "Policy not initialized.",
@@ -466,10 +506,9 @@ ifapi_policyeval_cbauth(
     }
     cb_ctx = current_policy->app_data;
 
-    do {
-        next_case = false;
-        switch (cb_ctx->cb_state) {
+    switch (cb_ctx->cb_state) {
         statecase(cb_ctx->cb_state, POL_CB_EXECUTE_INIT);
+            cb_ctx->flush_handle = false;
             cb_ctx->auth_index = ESYS_TR_NONE;
             /* Search object with name in keystore. */
             r = ifapi_keystore_search_obj(&fapi_ctx->keystore, &fapi_ctx->io,
@@ -508,30 +547,20 @@ ifapi_policyeval_cbauth(
                 cb_ctx->cb_state = POL_CB_AUTHORIZE_OBJECT;
 
                 cb_ctx->auth_object_ptr = &cb_ctx->auth_object;
-                next_case = true;
-                break;
+                return TSS2_FAPI_RC_TRY_AGAIN;
             } else if (cb_ctx->object.objectType == IFAPI_HIERARCHY_OBJ) {
                 cb_ctx->cb_state = POL_CB_AUTHORIZE_OBJECT;
                 cb_ctx->auth_object_ptr = &cb_ctx->object;
-                next_case = true;
-                break;
+                return TSS2_FAPI_RC_TRY_AGAIN;
             } else {
                 cb_ctx->key_handle = cb_ctx->object.handle;
-                cb_ctx->cb_state = POL_CB_LOAD_KEY;
+                if (cb_ctx->key_handle == ESYS_TR_NONE) {
+                    cb_ctx->cb_state = POL_CB_LOAD_KEY;
+                    return TSS2_FAPI_RC_TRY_AGAIN;
+                }
             }
             fallthrough;
 
-        statecase(cb_ctx->cb_state, POL_CB_LOAD_KEY);
-            /* Key loading and authorization */
-            r = ifapi_load_key(fapi_ctx, cb_ctx->object_path,
-                               &cb_ctx->auth_object_ptr);
-            FAPI_SYNC(r, "Fapi load key.", cleanup);
-
-            cb_ctx->object = *cb_ctx->key_object_ptr;
-            SAFE_FREE(cb_ctx->key_object_ptr);
-            cb_ctx->auth_object_ptr = &cb_ctx->object;
-            fallthrough;
-
         statecase(cb_ctx->cb_state, POL_CB_AUTHORIZE_OBJECT);
             r = ifapi_authorize_object(fapi_ctx, cb_ctx->auth_object_ptr, authSession);
             return_try_again(r);
@@ -541,9 +570,55 @@ ifapi_policyeval_cbauth(
             break;
             /* FALLTHRU */
 
+        statecase(cb_ctx->cb_state, POL_CB_LOAD_KEY);
+            /* Prepare new context for loadkey in policy and skip session creation. */
+            memset(&cb_ctx->load_ctx, 0, sizeof(IFAPI_LoadKey));
+            cb_ctx->load_ctx.prepare_state = PREPARE_LOAD_KEY_INIT_KEY;
+            memset(&cb_ctx->create_primary_ctx, 0, sizeof(IFAPI_CreatePrimary));
+            fallthrough;
+
+        statecase(cb_ctx->cb_state, POL_CB_LOAD_KEY_FINISH);
+            cb_ctx->load_ctx_sav = fapi_ctx->loadKey;
+            cb_ctx->create_primary_ctx_sav = fapi_ctx->createPrimary;
+            fapi_ctx->loadKey = cb_ctx->load_ctx;
+            fapi_ctx->createPrimary = cb_ctx->create_primary_ctx;
+            cb_ctx->auth_object_ptr = &cb_ctx->load_ctx.auth_object;
+            r = ifapi_load_key(fapi_ctx, ifapi_get_object_path(&cb_ctx->object),
+                               &cb_ctx->auth_object_ptr);
+            if (r == TSS2_RC_SUCCESS &&
+                !cb_ctx->load_ctx.auth_object.misc.key.persistent_handle) {
+                current_policy->flush_handle = true;
+            }
+            cb_ctx->load_ctx = fapi_ctx->loadKey;
+            cb_ctx->create_primary_ctx = fapi_ctx->createPrimary;
+            fapi_ctx->loadKey = cb_ctx->load_ctx_sav;
+            fapi_ctx->createPrimary = cb_ctx->create_primary_ctx_sav;
+            FAPI_SYNC(r, "Fapi load key.", cleanup);
+
+            ifapi_cleanup_ifapi_object(&cb_ctx->object);
+            cb_ctx->object = *cb_ctx->auth_object_ptr;
+            fallthrough;
+
+        statecase(cb_ctx->cb_state, POL_CB_AUTHORIZE_KEY);
+            cb_ctx->load_ctx_sav = fapi_ctx->loadKey;
+            cb_ctx->create_primary_ctx_sav = fapi_ctx->createPrimary;
+            fapi_ctx->loadKey = cb_ctx->load_ctx;
+            fapi_ctx->createPrimary = cb_ctx->create_primary_ctx;
+            cb_ctx->object = *cb_ctx->auth_object_ptr;
+            *auth_handle = cb_ctx->auth_object_ptr->handle;
+
+            r = ifapi_authorize_object(fapi_ctx, cb_ctx->auth_object_ptr, authSession);
+            return_try_again(r);
+
+            fapi_ctx->loadKey = cb_ctx->load_ctx_sav;
+            goto_if_error(r, "Authorize  object.", cleanup);
+
+            cb_ctx->cb_state = POL_CB_EXECUTE_INIT;
+            break;
+            /* FALLTHRU */
+
         statecasedefault(cb_ctx->cb_state);
-        }
-    } while (next_case);
+    }
     *object_handle = cb_ctx->object.handle;
     if (cb_ctx->object.objectType == IFAPI_NV_OBJ)
         *auth_handle = cb_ctx->auth_index;
