@@ -165,28 +165,30 @@ ifapi_profiles_initialize_finish(
     r = ifapi_io_read_finish(io, &buffer, NULL);
     return_if_error(r, "Reading profile failed");
 
-    jso = json_tokener_parse((char *) buffer);
+    jso = ifapi_parse_json((char *) buffer);
     free(buffer);
     if (jso == NULL) {
         LOG_ERROR("Failed to parse profile %s", profiles->filenames[profiles->profiles_idx]);
-        return TSS2_FAPI_RC_BAD_VALUE;
+        r = TSS2_FAPI_RC_BAD_VALUE;
+        goto error;
     }
 
     r = ifapi_profile_json_deserialize(jso,
             &profiles->profiles[profiles->profiles_idx].profile);
     json_object_put(jso);
-    return_if_error2(r, "Parsing profile %s failed",
-                     profiles->filenames[profiles->profiles_idx]);
+    goto_if_error2(r, "Parsing profile %s failed", error,
+                   profiles->filenames[profiles->profiles_idx]);
 
     r = ifapi_profile_checkpcrs(&profiles->profiles[profiles->profiles_idx].profile.pcr_selection);
-    return_if_error2(r, "Malformed profile pcr selection for profile %s",
-                     profiles->filenames[profiles->profiles_idx]);
+    goto_if_error2(r, "Malformed profile pcr selection for profile %s", error,
+                   profiles->filenames[profiles->profiles_idx]);
 
     profiles->profiles_idx += 1;
 
     if (profiles->profiles_idx < profiles->num_profiles) {
         r = ifapi_io_read_async(io, profiles->filenames[profiles->profiles_idx]);
-        return_if_error2(r, "Reading profile %s", profiles->filenames[profiles->profiles_idx]);
+        goto_if_error2(r, "Reading profile %s", error,
+                       profiles->filenames[profiles->profiles_idx]);
 
         return TSS2_FAPI_RC_TRY_AGAIN;
     }
@@ -201,7 +203,8 @@ ifapi_profiles_initialize_finish(
     if (i == profiles->num_profiles) {
         LOG_ERROR("Default profile %s not in the list of loaded profiles",
                   profiles->default_name);
-        return TSS2_FAPI_RC_BAD_VALUE;
+        r = TSS2_FAPI_RC_BAD_VALUE;
+        goto error;
     }
 
     for (i = 0; i < profiles->num_profiles; i++) {
@@ -210,6 +213,14 @@ ifapi_profiles_initialize_finish(
     SAFE_FREE(profiles->filenames);
 
     return TSS2_RC_SUCCESS;
+
+ error:
+    for (i = 0; i < profiles->num_profiles; i++) {
+        SAFE_FREE(profiles->filenames[i]);
+    }
+    SAFE_FREE(profiles->filenames);
+    ifapi_profiles_finalize(profiles);
+    return r;
 }
 
 /** Return the profile data for a given profile name.
@@ -239,7 +250,7 @@ ifapi_profiles_get(
     size_t len;
 
     /* if no name or nor profile prefix is given, use the default profile */
-    if (!name || strncmp(name, "P_", 2) != 0 || strncmp(name, "/P_", 2) != 0) {
+    if (!name || !(strncmp(name, "P_", 2) == 0 || strncmp(name, "/P_", 3) == 0)) {
         *profile = &profiles->default_profile;
         return TSS2_RC_SUCCESS;
     }
@@ -336,17 +347,18 @@ ifapi_profile_json_deserialize(
     };
 
     LOG_TRACE("call");
+    memset(out, 0, sizeof(IFAPI_PROFILE));
     return_if_null(out, "Bad reference.", TSS2_FAPI_RC_BAD_REFERENCE);
 
     if (!ifapi_get_sub_object(jso, "type", &jso2)) {
-        LOG_ERROR("Bad value");
+        LOG_ERROR("Field \"type\" not found.");
         return TSS2_FAPI_RC_BAD_VALUE;
     }
     r = ifapi_json_TPMI_ALG_PUBLIC_deserialize(jso2, &out->type);
-    return_if_error(r, "BAD VALUE");
+    return_if_error(r, "Bad value for field \"type\".");
 
     if (!ifapi_get_sub_object(jso, "srk_template", &jso2)) {
-        LOG_ERROR("Bad value");
+        LOG_ERROR("Field \"srk_template\" not found.");
         return TSS2_FAPI_RC_BAD_VALUE;
     }
     out->srk_template = strdup(json_object_get_string(jso2));
@@ -360,7 +372,7 @@ ifapi_profile_json_deserialize(
     }
 
     if (!ifapi_get_sub_object(jso, "ek_template", &jso2)) {
-        LOG_ERROR("Bad value");
+        LOG_ERROR("Field \"ek_template\" not found.");
         return TSS2_FAPI_RC_BAD_VALUE;
     }
     out->ek_template = strdup(json_object_get_string(jso2));
@@ -377,87 +389,87 @@ ifapi_profile_json_deserialize(
         memset(&out->ecc_signing_scheme, 0, sizeof(TPMT_SIG_SCHEME));
     } else {
         r = ifapi_json_TPMT_SIG_SCHEME_deserialize(jso2, &out->ecc_signing_scheme);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"ecc_signing_scheme\".");
     }
 
     if (!ifapi_get_sub_object(jso, "rsa_signing_scheme", &jso2)) {
         memset(&out->rsa_signing_scheme, 0, sizeof(TPMT_SIG_SCHEME));
     } else {
         r = ifapi_json_TPMT_SIG_SCHEME_deserialize(jso2, &out->rsa_signing_scheme);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"rsa_signing_scheme\".");
     }
 
     if (!ifapi_get_sub_object(jso, "rsa_decrypt_scheme", &jso2)) {
         memset(&out->rsa_decrypt_scheme, 0, sizeof(TPMT_RSA_DECRYPT));
     } else {
         r = ifapi_json_TPMT_RSA_DECRYPT_deserialize(jso2, &out->rsa_decrypt_scheme);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"rsa_decrypt_scheme\".");
     }
 
     if (!ifapi_get_sub_object(jso, "sym_mode", &jso2)) {
-        LOG_ERROR("Bad value");
+        LOG_ERROR("Field \"sym_mode\" not found.");
         return TSS2_FAPI_RC_BAD_VALUE;
     }
-    r = ifapi_json_TPMI_ALG_SYM_MODE_deserialize(jso2, &out->sym_mode);
-    return_if_error(r, "BAD VALUE");
+    r = ifapi_json_TPMI_ALG_CIPHER_MODE_deserialize(jso2, &out->sym_mode);
+    return_if_error(r, "Bad value for field \"sym_mode\".");
 
     if (!ifapi_get_sub_object(jso, "sym_parameters", &jso2)) {
-        LOG_ERROR("Bad value");
+        LOG_ERROR("Field \"sym_parameters\" not found.");
         return TSS2_FAPI_RC_BAD_VALUE;
     }
     r = ifapi_json_TPMT_SYM_DEF_OBJECT_deserialize(jso2, &out->sym_parameters);
-    return_if_error(r, "BAD VALUE");
+    return_if_error(r, "Bad value for field \"sym_parameters\".");
 
     if (!ifapi_get_sub_object(jso, "sym_block_size", &jso2)) {
-        LOG_ERROR("Bad value");
+        LOG_ERROR("Field \"sym_block_size\" not found.");
         return TSS2_FAPI_RC_BAD_VALUE;
     }
     r = ifapi_json_UINT16_deserialize(jso2, &out->sym_block_size);
-    return_if_error(r, "BAD VALUE");
+    return_if_error(r, "Bad value for field \"sym_block_size\".");
 
     if (!ifapi_get_sub_object(jso, "pcr_selection", &jso2)) {
-        LOG_ERROR("Bad value");
+        LOG_ERROR("Field \"pcr_selection\" not found.");
         return TSS2_FAPI_RC_BAD_VALUE;
     }
     r = ifapi_json_TPML_PCR_SELECTION_deserialize(jso2, &out->pcr_selection);
-    return_if_error(r, "BAD VALUE");
+    return_if_error(r, "Bad value for field \"pcr_selection\".");
 
     if (!ifapi_get_sub_object(jso, "nameAlg", &jso2)) {
-        LOG_ERROR("Bad value");
+        LOG_ERROR("Field \"nameAlg\" not found.");
         return TSS2_FAPI_RC_BAD_VALUE;
     }
     r = ifapi_json_TPMI_ALG_HASH_deserialize(jso2, &out->nameAlg);
-    return_if_error(r, "BAD VALUE");
+    return_if_error(r, "Bad value for field \"nameAlg\".");
 
     if (out->type == TPM2_ALG_RSA) {
         if (!ifapi_get_sub_object(jso, "exponent", &jso2)) {
-            LOG_ERROR("Bad value");
+            LOG_ERROR("Field \"exponent\" not found.");
             return TSS2_FAPI_RC_BAD_VALUE;
         }
         r = ifapi_json_UINT32_deserialize(jso2, &out->exponent);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"exponent\".");
         if (!ifapi_get_sub_object(jso, "keyBits", &jso2)) {
-            LOG_ERROR("Bad value");
+            LOG_ERROR("Field \"keyBits\" not found.");
             return TSS2_FAPI_RC_BAD_VALUE;
 
         }
         r = ifapi_json_TPMI_RSA_KEY_BITS_deserialize(jso2, &out->keyBits);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"keyBits\".");
 
     } else if (out->type == TPM2_ALG_ECC) {
         if (!ifapi_get_sub_object(jso, "curveID", &jso2)) {
-            LOG_ERROR("Bad value");
+            LOG_ERROR("Field \"curveID\" not found.");
             return TSS2_FAPI_RC_BAD_VALUE;
         }
         r = ifapi_json_TPMI_ECC_CURVE_deserialize(jso2, &out->curveID);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"curveID\".");
     }
 
     if (!ifapi_get_sub_object(jso, "session_symmetric", &jso2)) {
         out->session_symmetric = session_symmetric_default;
     } else {
         r = ifapi_json_TPMT_SYM_DEF_deserialize(jso2, &out->session_symmetric);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"session_symmetric\".");
     }
 
     if (ifapi_get_sub_object(jso, "eh_policy", &jso2)) {
@@ -509,21 +521,21 @@ ifapi_profile_json_deserialize(
         out->newMaxTries = 5;
     } else {
         r = ifapi_json_UINT32_deserialize(jso2, &out->newMaxTries);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"newMaxTries\".");
     }
 
     if (!ifapi_get_sub_object(jso, "newRecoveryTime", &jso2)) {
         out->newRecoveryTime = 1000;
     } else {
         r = ifapi_json_UINT32_deserialize(jso2, &out->newRecoveryTime);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"newRecoveryTime\".");
     }
 
     if (!ifapi_get_sub_object(jso, "lockoutRecovery", &jso2)) {
         out->lockoutRecovery = 1000;
     } else {
         r = ifapi_json_UINT32_deserialize(jso2, &out->lockoutRecovery);
-        return_if_error(r, "BAD VALUE");
+        return_if_error(r, "Bad value for field \"lockoutRecovery\".");
     }
 
     LOG_TRACE("true");
