@@ -16,7 +16,7 @@ int
 main (int argc, char *argv[])
 {
     TSS2_RC rc;
-    TSS2_SYS_CONTEXT *sapi_context;
+    TSS2_SYS_CONTEXT *sys_context;
 
     test_opts_t opts = {
         .tcti_type      = TCTI_DEFAULT,
@@ -29,16 +29,16 @@ main (int argc, char *argv[])
     if (sanity_check_test_opts (&opts) != 0)
         exit (1);
 
-    sapi_context = sapi_init_from_opts (&opts);
-    if (sapi_context == NULL)
+    sys_context = sys_init_from_opts (&opts);
+    if (sys_context == NULL)
         exit (1);
 
-    rc = Tss2_Sys_Startup(sapi_context, TPM2_SU_CLEAR);
+    rc = Tss2_Sys_Startup(sys_context, TPM2_SU_CLEAR);
     if (rc != TSS2_RC_SUCCESS && rc != TPM2_RC_INITIALIZE) {
         LOG_ERROR("TPM Startup FAILED! Response Code : 0x%x", rc);
         exit(1);
     }
 
-    sapi_teardown_full (sapi_context);
+    sys_teardown_full (sys_context);
     return 0;
 }
