@@ -22,7 +22,7 @@
  * The test can be executed with RSA or ECC keys. ECC will be used if
  * ECC is defined.
  *
- * Tested ESAPI commands:
+ * Tested ESYS commands:
  *  - Esys_CreatePrimary() (M)
  *  - Esys_FlushContext() (M)
  *  - Esys_StartAuthSession() (M)
@@ -42,10 +42,15 @@ test_esys_create_primary_hmac(ESYS_CONTEXT * esys_context)
     ESYS_TR session = ESYS_TR_NONE;
     TPMT_SYM_DEF symmetric = { .algorithm = TPM2_ALG_NULL };
 
+    TPM2B_PUBLIC *outPublic = NULL;
+    TPM2B_CREATION_DATA *creationData = NULL;
+    TPM2B_DIGEST *creationHash = NULL;
+    TPMT_TK_CREATION *creationTicket = NULL;
+
     r = Esys_StartAuthSession(esys_context, ESYS_TR_NONE, ESYS_TR_NONE,
                               ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                               NULL,
-                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA1,
+                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA256,
                               &session);
 
     goto_if_error(r, "Error: During initialization of session", error);
@@ -69,7 +74,7 @@ test_esys_create_primary_hmac(ESYS_CONTEXT * esys_context)
         .size = 0,
         .publicArea = {
             .type = TPM2_ALG_ECC,
-            .nameAlg = TPM2_ALG_SHA1,
+            .nameAlg = TPM2_ALG_SHA256,
             .objectAttributes = (TPMA_OBJECT_USERWITHAUTH |
                                  TPMA_OBJECT_RESTRICTED |
                                  TPMA_OBJECT_SIGN_ENCRYPT |
@@ -88,7 +93,7 @@ test_esys_create_primary_hmac(ESYS_CONTEXT * esys_context)
                  .scheme = {
                       .scheme = TPM2_ALG_ECDSA,
                       .details = {.ecdsa =
-                                  {.hashAlg = TPM2_ALG_SHA1}
+                                  {.hashAlg = TPM2_ALG_SHA256}
                       }
                   },
                  .curveID = TPM2_ECC_NIST_P256,
@@ -111,7 +116,7 @@ test_esys_create_primary_hmac(ESYS_CONTEXT * esys_context)
         .size = 0,
         .publicArea = {
             .type = TPM2_ALG_RSA,
-            .nameAlg = TPM2_ALG_SHA1,
+            .nameAlg = TPM2_ALG_SHA256,
             .objectAttributes = (TPMA_OBJECT_USERWITHAUTH |
                                  TPMA_OBJECT_RESTRICTED |
                                  TPMA_OBJECT_DECRYPT |
@@ -164,32 +169,30 @@ test_esys_create_primary_hmac(ESYS_CONTEXT * esys_context)
     goto_if_error(r, "Error: TR_SetAuth", error);
 
     RSRC_NODE_T *objectHandle_node;
-    TPM2B_PUBLIC *outPublic;
-    TPM2B_CREATION_DATA *creationData;
-    TPM2B_DIGEST *creationHash;
-    TPMT_TK_CREATION *creationTicket;
 
     r = Esys_CreatePrimary(esys_context, ESYS_TR_RH_OWNER, session,
                            ESYS_TR_NONE, ESYS_TR_NONE, &inSensitive, &inPublic,
                            &outsideInfo, &creationPCR, &objectHandle,
                            &outPublic, &creationData, &creationHash,
                            &creationTicket);
-    goto_if_error(r, "Error esapi create primary", error);
-
+    goto_if_error(r, "Error esys create primary", error);
     r = esys_GetResourceObject(esys_context, objectHandle,
                                &objectHandle_node);
     goto_if_error(r, "Error Esys GetResourceObject", error);
-    LOG_INFO("Created Primary with TPM handle 0x%08x...",
-             objectHandle_node->rsrc.handle);
+    ESYS_TR tpmHandle MAYBE_UNUSED = objectHandle_node->rsrc.handle;
+    LOG_INFO("Created Primary with TPM handle 0x%08x...", tpmHandle);
 
     r = Esys_FlushContext(esys_context, objectHandle);
     goto_if_error(r, "Error during FlushContext", error);
 
-    LOG_INFO("Done with handle 0x%08x...", objectHandle_node->rsrc.handle);
-
+    LOG_INFO("Done with handle 0x%08x...", tpmHandle);
     r = Esys_FlushContext(esys_context, session);
     goto_if_error(r, "Flushing context", error);
 
+    Esys_Free(outPublic);
+    Esys_Free(creationData);
+    Esys_Free(creationHash);
+    Esys_Free(creationTicket);
     return EXIT_SUCCESS;
 
  error:
@@ -207,11 +210,14 @@ test_esys_create_primary_hmac(ESYS_CONTEXT * esys_context)
         }
     }
 
-
+    Esys_Free(outPublic);
+    Esys_Free(creationData);
+    Esys_Free(creationHash);
+    Esys_Free(creationTicket);
     return EXIT_FAILURE;
 }
 
 int
-test_invoke_esapi(ESYS_CONTEXT * esys_context) {
+test_invoke_esys(ESYS_CONTEXT * esys_context) {
     return test_esys_create_primary_hmac(esys_context);
 }
