@@ -29,12 +29,6 @@
 /* global TCTI object, use to return reference from */
 static TSS2_TCTI_CONTEXT_COMMON_V2 tcti_instance = { 0, };
 
-char *
-__wrap_dlerror(void)
-{
-    return (char*)__func__;
-}
-
 void *
 __wrap_dlopen(const char *filename, int flags)
 {
@@ -61,6 +55,12 @@ __wrap_dlsym(void *handle, const char *symbol)
     return mock_type(void *);
 }
 
+void *
+__wrap___dlsym_time64(void *handle, const char *symbol)
+{
+    return __wrap_dlsym(handle, symbol);
+}
+
 TSS2_TCTI_INFO *
 __wrap_Tss2_Tcti_Fake_Info(void)
 {
